/*lcdprint  v0.1            (c)2001 by Andreas Schwarz (andreas-s@web.de)
 *Prints on a 2x16 LCD connected to parallel port
 *
 *
 *Usage: lcd.exe [options] text...
 *Options:
 *  -i              Init LCD
 *  -s [char line]  Set cursor position
 *  -c              Clear Display
 *  -h              Cursor Home
 *  -?              This usage.
 */

#include <conio.h>
#include <stdio.h>
#include <time.h>
#include "ztimer/include/debug.h"
#include "ztimer/include/pmode.h"
#include "ztimer/include/ztimer.h"

#define Base    0x378
#define Data    Base
#define Status  Base+1
#define Control Base+2

static int  lcd_x, lcd_y;

int do_init=0, do_set=0, do_new=0, do_clear=0, do_home=0;
unsigned char go_x, go_y;

int position=0;

const char USAGE[] =
"lcdprint  v0.1            (c)2001 by Andreas Schwarz (andreas-s@web.de)\n"
"Prints on a 2x16 LCD connected to parallel port\n\n"
"Usage: lcd.exe [options] text...\n"
"Options:\n"
"  -i              Init LCD\n"
"  -s [char line]  Set cursor position\n"
"  -c              Clear Display\n"
"  -h              Cursor Home\n"
"  -?              This usage.\n\n";


int ShowUsage( char *szError ) {

  fprintf( stderr, USAGE );

  fprintf(stderr, szError);

  return 0;
}


void waitms(unsigned long time) {

   unsigned long oldtime = LZTimerLap();
   while(LZTimerLap() < oldtime + time*1000);

}

void lcd_write_command(int value) {
        outp(Data, value);
        outp(Control, 0x03);
        outp(Control, 0x02);
        waitms(1);
        outp(Control, 0x03);
}

void lcd_write_data(int value) {
        outp(Data, value);
        outp(Control, 0x01);
        outp(Control, 0x00);
        waitms(1);
        outp(Control, 0x01);
}

static void lcd_goto(void)
/* goto position (lcd_x,lcd_y) */
{
    lcd_write_command((1<<7)+0x40*lcd_y+lcd_x);
}

void lcd_gotoxy(int x, int y)
/* goto position (x,y) */
{
    lcd_x = x; lcd_y = y;
    lcd_goto();
}

static void lcd_newline(void)
/* goto start of next line */
{
    lcd_x = 0;
    if (++lcd_y >= 2)
        lcd_y = 0;
}

void lcd_clrscr(void)
/* clear lcd */
{
    lcd_x = lcd_y = 0;
    lcd_write_command(1<<0);
}

void lcd_home(void)
/* set cursor to home position */
{
    lcd_x = lcd_y = 0;
    lcd_write_command(1<<1);
}

void lcd_putchar(char data)
/* print character to current cursor position */
{
    if (data=='\n') {
        lcd_newline();
        lcd_goto();
    }
    else {
        if (++lcd_x >= 17) {
            lcd_newline();
            lcd_goto();
        }
        lcd_write_data(data);
    }
}

void lcd_puts(char s[])
/* print string on lcd (no auto linefeed) */
{
    char *c;

    if (!(c=s)) return;

    while (*c) {
        lcd_putchar(*c);
        c++;
    }
}

void lcd_init(void)
{
        lcd_write_command(0x38);
        lcd_write_command(0x01);
        lcd_write_command(0x06);
        lcd_write_command(0x0C);
}

int main( int argc, char *argv[] )
{
   int i;

   ZTimerInit();
   if (argc == 1) {
      return ShowUsage("");
   }

   for (i=1; i<argc; i++)
    {

      if (argv[i][0] != '-') {
         position = i;
         break;
      }

      switch ( argv[i][1] )
        {
          case 'i':
            do_init=1;
            break;

          case 's':
            if ((i+2 >= argc)) //mindestens 2 nachfolgende Parameter?
               return ShowUsage("Error: Missing parameter for -s\n");
            if (argv[i+1][1]!=0) {
               go_x = 10 * (argv[i+1][0] - 48) + (argv[i+1][1] - 48); //wenn x zweistellig ist
            } else {
               go_x = argv[i+1][0] - 48; //wenn x einstellig ist
            }
            go_y = argv[i+2][0] - 48; //y muss immer einstellig sein
            i+=2; //die 2 nachfolgenden Parameter mssen nicht mehr verarbeitet werden
            do_set=1;
            break;

          case 'c':
            do_clear=1;
            break;

          case 'h':
            do_home=1;
            break;

          case '?':
            return ShowUsage("");

          default:
            return ShowUsage("Error: Invalid option\n");
        }
   }

   if(do_init==1) {
      lcd_init();
      waitms(10);
   }

   if(do_clear==1) {
      lcd_clrscr();
      waitms(5);
   }

   if(do_home==1) {
      lcd_home();
      waitms(5);
   }

   if(do_set==1) {
      //lcd_x = go_x-1; lcd_y = go_y-1;
      //lcd_goto();
      lcd_gotoxy(go_x-1, go_y-1);
      waitms(5);
   }

   if (position!=0) {
      for(i=position; i<argc; i++) {
         lcd_puts(argv[i]);
         if (i+1<argc)
            lcd_putchar(' ');
      }
   }

    return 0;
}
